﻿using Borepin.Service.ErrorMessage;
using NFC.Cards.NXP_MIFARE_DESFire;
using NFC.Cards.NXP_MIFARE_DESFire.Enums;
using NFC.Helper;
using NFC.Helper.Crypto;
using NFC.Interfaces;
using System;
using ZXing.Aztec.Internal;

namespace Borepin.Model
{
    public class FabFireCard
    {
        #region Private Fields
        INFCService _NFCService;
        IErrorMessageService _ErrorMessageService;
        #endregion

        #region Constructors
        public FabFireCard(INFCService nfcService, IErrorMessageService errorMessageService)
        {
            _NFCService = nfcService;
            _ErrorMessageService = errorMessageService;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Format Card
        /// </summary>
        /// <param name="readerID"></param>
        /// <param name="cardConfig"></param>
        public void FormatCard(string readerID, CardConfig cardConfig)
        {
            _NFCService.Connect(readerID);

            NXP_MIFARE_DESFire card = new NXP_MIFARE_DESFire(_NFCService);
            card.SelectApplication(0x000000);
            card.AuthenticateISO_DES(0x00, cardConfig.PICCKey);
            card.Format();

            _NFCService.Disconnect();
        }

        /// <summary>
        /// Create DESFire Card for V1.0
        /// </summary>
        /// <param name="readerID"></param>
        /// <param name="cardConfig"></param>
        /// <returns>Key #1 for authentication </returns>
        public byte[] CreateCard(string readerID, CardConfig cardConfig)
        {
            _NFCService.Connect(readerID);

            NXP_MIFARE_DESFire card = new NXP_MIFARE_DESFire(_NFCService);

            card.SelectApplication(0x000000);

            if(cardConfig.DoFormat)
            {
                card.AuthenticateISO_DES(0x00, new CipherKey(CipherType.TDES));
            }
            else
            {
                card.AuthenticateISO_DES(0x00, cardConfig.PICCKey);
            }

            byte keySettings1 = card.GenerateKeySetting1
                (
                0x00,
                ChangeMasterKeySettings.WITHMASTERKEY,
                CreateDeleteFile.NOKEY,
                FileDirectoryAccess.NOKEY,
                ChangeMasterKey.CHANGEABLE
                );

            byte keySettings2 = card.GenerateKeySetting2
                (
                CryptoOperationsType.AES,
                FileIdentifies.NOTUSED,
                0x02
                );

            card.CreateApplication(0x464142, keySettings1, keySettings2);

            card.SelectApplication(0x464142);
            card.AuthenticateISO_AES(0x00, new CipherKey(CipherType.AES));
            ushort file_access = card.GenerateFileAccessRights
                (
                FileAccessRights.FREE,
                0x00,
                0x00,
                0x00
                );
            card.CreateFile_Standard(0x01, FileCommunication.PLAIN, file_access, cardConfig.MetaInfo.Length);
            card.CreateFile_Standard(0x02, FileCommunication.PLAIN, file_access, cardConfig.SpaceInfo.Length);
            card.CreateFile_Standard(0x03, FileCommunication.PLAIN, file_access, cardConfig.CardToken.Length);

            card.WriteData(0x00, 0x00, cardConfig.MetaInfo);
            card.WriteData(0x00, 0x00, cardConfig.SpaceInfo);
            card.WriteData(0x00, 0x00, cardConfig.CardToken);

            auth_key = cardConfig.GenerateRandomKey();

            card.ChangeOtherKey_AES(0x01, auth_key, new CipherKey(CipherType.AES), 0x00);
            card.ChangeKey_AES(0x00, cardConfig.APPKey, 0x00);

            _NFCService.Disconnect();

            return auth_key;
        }
        #endregion
    }
}
